@@ -34,7 +34,8 @@
 #include <thrust/system_error.h>
 #include <thrust/system/cuda/error.h>
 
-THRUST_BEGIN_NS
+namespace thrust
+{
 
 namespace cuda_cub {
 
@@ -148,11 +149,15 @@ trivial_copy_device_to_device(Policy &    policy,
 inline void __host__ __device__
 terminate()
 {
-#ifdef __CUDA_ARCH__
-  asm("trap;");
-#else
-  std::terminate();
-#endif
+  if (THRUST_IS_DEVICE_CODE) {
+    #if THRUST_INCLUDE_DEVICE_CODE
+      asm("trap;");
+    #endif
+  } else {
+    #if THRUST_INCLUDE_HOST_CODE
+      std::terminate();
+    #endif
+  }
 }
 
 __host__  __device__
@@ -166,19 +171,23 @@ inline void throw_on_error(cudaError_t status)
 
   if (cudaSuccess != status)
   {
-#if !defined(__CUDA_ARCH__)
-    throw thrust::system_error(status, thrust::cuda_category());
-#else
-#if __THRUST_HAS_CUDART__
-    printf("Thrust CUDA backend error: %s: %s\n",
-           cudaGetErrorName(status),
-           cudaGetErrorString(status));
-#else
-    printf("Thrust CUDA backend error: %d\n",
-           static_cast<int>(status));
-#endif
-    cuda_cub::terminate();
-#endif
+    if (THRUST_IS_HOST_CODE) {
+      #if THRUST_INCLUDE_HOST_CODE
+        throw thrust::system_error(status, thrust::cuda_category());
+      #endif
+    } else {
+      #if THRUST_INCLUDE_DEVICE_CODE
+        #if __THRUST_HAS_CUDART__
+          printf("Thrust CUDA backend error: %s: %s\n",
+                 cudaGetErrorName(status),
+                 cudaGetErrorString(status));
+        #else
+          printf("Thrust CUDA backend error: %d\n",
+                 static_cast<int>(status));
+        #endif
+        cuda_cub::terminate();
+      #endif
+    }
   }
 }
 
@@ -193,21 +202,25 @@ inline void throw_on_error(cudaError_t status, char const *msg)
 
   if (cudaSuccess != status)
   {
-#if !defined(__CUDA_ARCH__)
-    throw thrust::system_error(status, thrust::cuda_category(), msg);
-#else
-#if __THRUST_HAS_CUDART__
-    printf("Thrust CUDA backend error: %s: %s: %s\n",
-           cudaGetErrorName(status),
-           cudaGetErrorString(status),
-           msg);
-#else
-    printf("Thrust CUDA backend error: %d: %s \n",
-           static_cast<int>(status),
-           msg);
-#endif
-    cuda_cub::terminate();
-#endif
+    if (THRUST_IS_HOST_CODE) {
+      #if THRUST_INCLUDE_HOST_CODE
+        throw thrust::system_error(status, thrust::cuda_category(), msg);
+      #endif
+    } else {
+      #if THRUST_INCLUDE_DEVICE_CODE
+        #if __THRUST_HAS_CUDART__
+          printf("Thrust CUDA backend error: %s: %s: %s\n",
+                 cudaGetErrorName(status),
+                 cudaGetErrorString(status),
+                 msg);
+        #else
+          printf("Thrust CUDA backend error: %d: %s \n",
+                 static_cast<int>(status),
+                 msg);
+        #endif
+        cuda_cub::terminate();
+      #endif
+    }
   }
 }
 
@@ -232,6 +245,18 @@ struct transform_input_iterator_t
   transform_input_iterator_t(InputIt input, UnaryOp op)
       : input(input), op(op) {}
 
+#if THRUST_CPP_DIALECT >= 2011
+  transform_input_iterator_t(const self_t &) = default;
+#endif
+
+  // UnaryOp might not be copy assignable, such as when it is a lambda.  Define
+  // an explicit copy assignment operator that doesn't try to assign it.
+  self_t& operator=(const self_t& o)
+  {
+    input = o.input;
+    return *this;
+  }
+
   /// Postfix increment
   __host__ __device__ __forceinline__ self_t operator++(int)
   {
@@ -350,6 +375,19 @@ struct transform_pair_of_input_iterators_t
                                       BinaryOp op_)
       : input1(input1_), input2(input2_), op(op_) {}
 
+#if THRUST_CPP_DIALECT >= 2011
+  transform_pair_of_input_iterators_t(const self_t &) = default;
+#endif
+
+  // BinaryOp might not be copy assignable, such as when it is a lambda.
+  // Define an explicit copy assignment operator that doesn't try to assign it.
+  self_t& operator=(const self_t& o)
+  {
+    input1 = o.input1;
+    input2 = o.input2;
+    return *this;
+  }
+
   /// Postfix increment
   __host__ __device__ __forceinline__ self_t operator++(int)
   {
@@ -864,4 +902,4 @@ struct counting_iterator_t
 
 }    // cuda_
 
-THRUST_END_NS
+} // end namespace thrust
